@@ -38,6 +38,9 @@ namespace Tomboy
 		public const string SYNC_LOCAL_PATH = "/apps/tomboy/sync/sync_local_path";
 		public const string SYNC_SELECTED_SERVICE_ADDIN = "/apps/tomboy/sync/sync_selected_service_addin";
 		public const string SYNC_CONFIGURED_CONFLICT_BEHAVIOR = "/apps/tomboy/sync/sync_conflict_behavior";
+		public const string SYNC_AUTOSYNC_TIMEOUT = "/apps/tomboy/sync/autosync_timeout";
+
+		public const string NOTE_RENAME_BEHAVIOR = "/apps/tomboy/note_rename_behavior";
 
 		public const string INSERT_TIMESTAMP_FORMAT = "/apps/tomboy/insert_timestamp/format";
 		
@@ -133,6 +136,12 @@ namespace Tomboy
 			case SYNC_CONFIGURED_CONFLICT_BEHAVIOR:
 				return 0;
 
+			case SYNC_AUTOSYNC_TIMEOUT:
+				return -1;
+
+			case NOTE_RENAME_BEHAVIOR:
+				return 0;
+
 			case INSERT_TIMESTAMP_FORMAT:
 				return Catalog.GetString ("dddd, MMMM d, h:mm tt");
 			}
