@@ -1,8 +1,10 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.Runtime.InteropServices;
 using System.Text;
+using System.Threading;
 using System.IO;
 using System.Xml;
 
@@ -60,26 +62,26 @@ namespace Tomboy
 		// calling this.
 		public static void PopupMenu (Gtk.Menu menu, Gdk.EventButton ev)
 		{
-			menu.Deactivated += DeactivateMenu;
-			menu.Popup (null,
-			            null,
-			            new Gtk.MenuPositionFunc (GetMenuPosition),
-			            (ev == null) ? 0 : ev.Button,
-			            (ev == null) ? Gtk.Global.CurrentEventTime : ev.Time);
-
-			// Highlight the parent
-			if (menu.AttachWidget != null)
-				menu.AttachWidget.State = Gtk.StateType.Selected;
+			PopupMenu (menu, ev, new Gtk.MenuPositionFunc (GetMenuPosition));
 		}
 
 		public static void PopupMenu (Gtk.Menu menu, Gdk.EventButton ev, Gtk.MenuPositionFunc mpf)
 		{
 			menu.Deactivated += DeactivateMenu;
-			menu.Popup (null,
-			            null,
-			            mpf,
-			            (ev == null) ? 0 : ev.Button,
-			            (ev == null) ? Gtk.Global.CurrentEventTime : ev.Time);
+			try {
+				menu.Popup (null,
+				            null,
+				            mpf,
+				            (ev == null) ? 0 : ev.Button,
+				            (ev == null) ? Gtk.Global.CurrentEventTime : ev.Time);
+			} catch {
+				Logger.Debug ("Menu popup failed with custom MenuPositionFunc; trying again without");
+				menu.Popup (null,
+				            null,
+				            null,
+				            (ev == null) ? 0 : ev.Button,
+				            (ev == null) ? Gtk.Global.CurrentEventTime : ev.Time);
+			}
 
 			// Highlight the parent
 			if (menu.AttachWidget != null)
@@ -103,7 +105,7 @@ namespace Tomboy
 				return ret.ScaleSimple (size, size, Gdk.InterpType.Bilinear);
 			} catch (ArgumentException) {}
 
-			Logger.Log ("Unable to load icon '{0}'.", resource_name);
+			Logger.Debug ("Unable to load icon '{0}'.", resource_name);
 			return null;
 		}
 
@@ -130,22 +132,18 @@ namespace Tomboy
 			return MakeImageButton (image, label);
 		}
 
-		public static void ShowHelp (string filename,
-		                             string link_id,
+		public static void ShowHelp (string help_uri,
 		                             Gdk.Screen screen,
 		                             Gtk.Window parent)
 		{
 			try {
-				Services.NativeApplication.DisplayHelp (
-					filename,
-					link_id,
-					screen);
+				Services.NativeApplication.DisplayHelp (help_uri, screen);
 			} catch {
-			string message =
-			Catalog.GetString ("The \"Tomboy Notes Manual\" could " +
-			"not be found.  Please verify " +
-			"that your installation has been " +
-			"completed successfully.");
+				string message =
+					Catalog.GetString ("The \"Tomboy Notes Manual\" could " +
+					"not be found.  Please verify " +
+					"that your installation has been " +
+					"completed successfully.");
 				HIGMessageDialog dialog =
 				        new HIGMessageDialog (parent,
 				                              Gtk.DialogFlags.DestroyWithParent,
@@ -157,6 +155,21 @@ namespace Tomboy
 				dialog.Destroy ();
 			}
 		}
+		
+		public static void ShowOpeningLocationError (Gtk.Window parent, string url, string error)
+		{
+			string message = String.Format ("{0}: {1}", url, error);
+
+			HIGMessageDialog dialog =
+			        new HIGMessageDialog (parent,
+			                              Gtk.DialogFlags.DestroyWithParent,
+			                              Gtk.MessageType.Info,
+			                              Gtk.ButtonsType.Ok,
+			                              Catalog.GetString ("Cannot open location"),
+			                              message);
+			dialog.Run ();
+			dialog.Destroy ();
+		}
 
 		/// <summary>
 		/// Get a string that is more friendly/pretty for the specified date.
@@ -186,10 +199,14 @@ namespace Tomboy
 				else if (date.DayOfYear < now.DayOfYear
 				                && date.DayOfYear > now.DayOfYear - 6)
 					pretty_str = show_time ?
-					             String.Format (Catalog.GetString ("{0} days ago, {1}"),
-					                            now.DayOfYear - date.DayOfYear, short_time) :
-					             String.Format (Catalog.GetString ("{0} days ago"),
-					                            now.DayOfYear - date.DayOfYear);
+					             String.Format (Catalog.GetPluralString (
+								"{0} day ago, {1}", "{0} days ago, {1}", 
+								now.DayOfYear - date.DayOfYear),
+								now.DayOfYear - date.DayOfYear, short_time) :
+					             String.Format (Catalog.GetPluralString (
+								"{0} day ago", "{0} days ago",
+								now.DayOfYear - date.DayOfYear),
+								now.DayOfYear - date.DayOfYear);
 				else if (date.DayOfYear > now.DayOfYear
 				                && date.DayOfYear == now.DayOfYear + 1)
 					pretty_str = show_time ?
@@ -199,10 +216,14 @@ namespace Tomboy
 				else if (date.DayOfYear > now.DayOfYear
 				                && date.DayOfYear < now.DayOfYear + 6)
 					pretty_str = show_time ?
-					             String.Format (Catalog.GetString ("In {0} days, {1}"),
-					                            date.DayOfYear - now.DayOfYear, short_time) :
-					             String.Format (Catalog.GetString ("In {0} days"),
-					                            date.DayOfYear - now.DayOfYear);
+					             String.Format (Catalog.GetPluralString (
+								"In {0} day, {1}", "In {0} days, {1}",
+								date.DayOfYear - now.DayOfYear),
+								date.DayOfYear - now.DayOfYear, short_time) :
+					             String.Format (Catalog.GetPluralString (
+								"In {0} day", "In {0} days",
+								date.DayOfYear - now.DayOfYear),
+								date.DayOfYear - now.DayOfYear);
 				else
 					pretty_str = show_time ?
 					             date.ToString (Catalog.GetString ("MMMM d, h:mm tt")) :
@@ -216,6 +237,31 @@ namespace Tomboy
 
 			return pretty_str;
 		}
+
+		/// <summary>
+		/// Invoke a method on the GUI thread, and wait for it to
+		/// return. If the method raises an exception, it will be
+		/// thrown from this method.
+		/// </summary>
+		/// <param name="a">
+		/// The action to invoke.
+		/// </param>
+		public static void GtkInvokeAndWait (Action a)
+		{
+			Exception mainThreadException = null;
+			AutoResetEvent evt = new AutoResetEvent (false);
+			Gtk.Application.Invoke (delegate {
+				try {
+					a.Invoke ();
+				} catch (Exception e) {
+					mainThreadException = e;
+				}
+				evt.Set ();
+			});
+			evt.WaitOne ();
+			if (mainThreadException != null)
+				throw mainThreadException;
+		}
 	}
 
 	public class GlobalKeybinder
@@ -439,7 +485,7 @@ namespace Tomboy
 		}
 	}
 
-	public class UriList : ArrayList
+	public class UriList : List<Uri>
 	{
 		public UriList (Note [] notes)
 		{
@@ -821,7 +867,6 @@ namespace Tomboy
 	class ToolMenuButton : Gtk.ToggleToolButton
 	{
 		Gtk.Menu menu;
-		Gtk.Image image;
 
 		public ToolMenuButton (Gtk.Toolbar toolbar,
 		                       string stock_image,
@@ -934,4 +979,21 @@ namespace Tomboy
 			}
 		}
 	}
+
+	public static class IOUtils
+	{
+		/// <summary>
+		/// Recursively copy the directory specified by old_path to
+		/// new_path. Assumes that old_path is an existing directory
+		/// and new_path does not exist.
+		/// </summary>
+		public static void CopyDirectory (string old_path, string new_path)
+		{
+			Directory.CreateDirectory (new_path);
+			foreach (string file_path in Directory.GetFiles (old_path))
+				File.Copy (file_path, Path.Combine (new_path, Path.GetFileName (file_path)));
+			foreach (string dir_path in Directory.GetDirectories (old_path))
+				CopyDirectory (dir_path, Path.Combine (new_path, Path.GetFileName (dir_path)));
+		}
+	}
 }
