@@ -1,7 +1,10 @@
+
+#pragma warning disable 169 // Unused private members
+
 using System;
 using System.Runtime.InteropServices;
 
-namespace PanelApplet
+namespace _Gnome
 {
 
 	public delegate void ContextMenuItemCallback ();
